"""
I won't pretend like this is best practice, but in creating animations for a video,
it can be very nice to simply have all of the Mobjects, Animations, Scenes, etc.
of manim available without having to worry about what namespace they come from.

Rather than having a large pile of "from <module> import *" at the top of every such
script, the intent of this file is to make it so that one can just include
"from manimlib.imports import *".  The effects of adding more modules
or refactoring the library on current or older scene scripts should be entirely
addressible by changing this file.

Note: One should NOT import from this file for main library code, it is meant only
as a convenience for scripts creating scenes for videos.
"""


from manimlib.constants import *

from manimlib.animation.animation import *
from manimlib.animation.composition import *
from manimlib.animation.creation import *
from manimlib.animation.fading import *
from manimlib.animation.growing import *
from manimlib.animation.indication import *
from manimlib.animation.movement import *
from manimlib.animation.numbers import *
from manimlib.animation.rotation import *
from manimlib.animation.specialized import *
from manimlib.animation.transform import *
from manimlib.animation.update import *

from manimlib.camera.camera import *
from manimlib.camera.mapping_camera import *
from manimlib.camera.moving_camera import *
from manimlib.camera.three_d_camera import *

from manimlib.mobject.coordinate_systems import *
from manimlib.mobject.changing import *
from manimlib.mobject.frame import *
from manimlib.mobject.functions import *
from manimlib.mobject.geometry import *
from manimlib.mobject.matrix import *
from manimlib.mobject.mobject import *
from manimlib.mobject.number_line import *
from manimlib.mobject.numbers import *
from manimlib.mobject.probability import *
from manimlib.mobject.shape_matchers import *
from manimlib.mobject.svg.brace import *
from manimlib.mobject.svg.drawings import *
from manimlib.mobject.svg.svg_mobject import *
from manimlib.mobject.svg.tex_mobject import *
from manimlib.mobject.svg.text_mobject import *
from manimlib.mobject.svg.code_mobject import *
from manimlib.mobject.three_d_utils import *
from manimlib.mobject.three_dimensions import *
from manimlib.mobject.types.image_mobject import *
from manimlib.mobject.types.point_cloud_mobject import *
from manimlib.mobject.types.vectorized_mobject import *
from manimlib.mobject.mobject_update_utils import *
from manimlib.mobject.value_tracker import *
from manimlib.mobject.vector_field import *

<<<<<<< HEAD
#from manimlib.for_3b1b_videos.common_scenes import *
#from manimlib.for_3b1b_videos.pi_creature import *
#from manimlib.for_3b1b_videos.pi_creature_animations import *
#from manimlib.for_3b1b_videos.pi_creature_scene import *

#from manimlib.once_useful_constructs.arithmetic import *
#from manimlib.once_useful_constructs.combinatorics import *
#from manimlib.once_useful_constructs.complex_transformation_scene import *
#from manimlib.once_useful_constructs.counting import *
#from manimlib.once_useful_constructs.fractals import *
#from manimlib.once_useful_constructs.graph_theory import *
#from manimlib.once_useful_constructs.light import *
=======
from manimlib.for_3b1b_videos.common_scenes import *
from manimlib.for_3b1b_videos.pi_creature import *
from manimlib.for_3b1b_videos.pi_creature_animations import *
from manimlib.for_3b1b_videos.pi_creature_scene import *

from manimlib.once_useful_constructs.arithmetic import *
from manimlib.once_useful_constructs.combinatorics import *
from manimlib.once_useful_constructs.complex_transformation_scene import *
from manimlib.once_useful_constructs.counting import *
from manimlib.once_useful_constructs.fractals import *
from manimlib.once_useful_constructs.graph_theory import *
from manimlib.once_useful_constructs.light import *
>>>>>>> 9a3d9e8fec15fb8887b558f1a331e6de84e47b2a

from manimlib.scene.graph_scene import *
from manimlib.scene.moving_camera_scene import *
from manimlib.scene.reconfigurable_scene import *
from manimlib.scene.scene import *
from manimlib.scene.sample_space_scene import *
from manimlib.scene.graph_scene import *
from manimlib.scene.scene_from_video import *
from manimlib.scene.three_d_scene import *
from manimlib.scene.vector_space_scene import *
from manimlib.scene.zoomed_scene import *

from manimlib.utils.bezier import *
from manimlib.utils.color import *
from manimlib.utils.config_ops import *
from manimlib.utils.debug import *
from manimlib.utils.images import *
from manimlib.utils.iterables import *
from manimlib.utils.file_ops import *
from manimlib.utils.paths import *
from manimlib.utils.rate_functions import *
from manimlib.utils.simple_functions import *
from manimlib.utils.sounds import *
from manimlib.utils.space_ops import *
from manimlib.utils.strings import *

# Non manim libraries that are also nice to have without thinking

import inspect
import itertools as it
import numpy as np
import operator as op
import os
import random
import re
import string
import sys
import math

from PIL import Image
from colour import Color



import numpy as np
import os

MEDIA_DIR = ""
VIDEO_DIR = ""
VIDEO_OUTPUT_DIR = ""
TEX_DIR = ""
TEXT_DIR = ""


def initialize_directories(config):
    global MEDIA_DIR
    global VIDEO_DIR
    global VIDEO_OUTPUT_DIR
    global TEX_DIR
    global TEXT_DIR

    video_path_specified = config["video_dir"] or config["video_output_dir"]

    if not (video_path_specified and config["tex_dir"]):
        if config["media_dir"]:
            MEDIA_DIR = config["media_dir"]
        else:
            MEDIA_DIR = os.path.join(
                os.path.expanduser('~'),
                "Dropbox (3Blue1Brown)/3Blue1Brown Team Folder"
            )
        if not os.path.isdir(MEDIA_DIR):
            MEDIA_DIR = os.path.join(os.getcwd(), "media")
        print(
            f"Media will be written to {MEDIA_DIR + os.sep}. You can change "
            "this behavior with the --media_dir flag."
        )
    else:
        if config["media_dir"]:
            print(
                "Ignoring --media_dir, since both --tex_dir and a video "
                "directory were both passed"
            )

    TEX_DIR = config["tex_dir"] or os.path.join(MEDIA_DIR, "Tex")
    TEXT_DIR = os.path.join(MEDIA_DIR, "texts")
    if not video_path_specified:
        VIDEO_DIR = os.path.join(MEDIA_DIR, "videos")
        VIDEO_OUTPUT_DIR = os.path.join(MEDIA_DIR, "videos")
    elif config["video_output_dir"]:
        VIDEO_OUTPUT_DIR = config["video_output_dir"]
    else:
        VIDEO_DIR = config["video_dir"]

    for folder in [VIDEO_DIR, VIDEO_OUTPUT_DIR, TEX_DIR, TEXT_DIR]:
        if folder != "" and not os.path.exists(folder):
            os.makedirs(folder)

NOT_SETTING_FONT_MSG='''
Warning:
You haven't set font.
If you are not using English, this may cause text rendering problem.
You set font like:
text = Text('your text', font='your font')
or:
class MyText(Text):
    CONFIG = {
        'font': 'My Font'
    }
'''
START_X = 30
START_Y = 20
NORMAL = 'NORMAL'
ITALIC = 'ITALIC'
OBLIQUE = 'OBLIQUE'
BOLD = 'BOLD'

TEX_USE_CTEX = False
TEX_TEXT_TO_REPLACE = "YourTextHere"
TEMPLATE_TEX_FILE = os.path.join(
    os.path.dirname(os.path.realpath(__file__)),
    "tex_template.tex" if not TEX_USE_CTEX else "ctex_template.tex"
)
with open(TEMPLATE_TEX_FILE, "r") as infile:
    TEMPLATE_TEXT_FILE_BODY = infile.read()
    TEMPLATE_TEX_FILE_BODY = TEMPLATE_TEXT_FILE_BODY.replace(
        TEX_TEXT_TO_REPLACE,
        "\\begin{align*}\n" + TEX_TEXT_TO_REPLACE + "\n\\end{align*}",
    )

HELP_MESSAGE = """
   Usage:
   python extract_scene.py <module> [<scene name>]
   -p preview in low quality
   -s show and save picture of last frame
   -w write result to file [this is default if nothing else is stated]
   -o <file_name> write to a different file_name
   -l use low quality
   -m use medium quality
   -a run and save every scene in the script, or all args for the given scene
   -q don't print progress
   -f when writing to a movie file, export the frames in png sequence
   -t use transperency when exporting images
   -n specify the number of the animation to start from
   -r specify a resolution
   -c specify a background color
"""
SCENE_NOT_FOUND_MESSAGE = """
   {} is not in the script
"""
CHOOSE_NUMBER_MESSAGE = """
Choose number corresponding to desired scene/arguments.
(Use comma separated list for multiple entries)
Choice(s): """
INVALID_NUMBER_MESSAGE = "Fine then, if you don't want to give a valid number I'll just quit"

NO_SCENE_MESSAGE = """
   There are no scenes inside that module
"""

# There might be other configuration than pixel shape later...
PRODUCTION_QUALITY_CAMERA_CONFIG = {
    "pixel_height": 1440,
    "pixel_width": 2560,
    "frame_rate": 60,
}

HIGH_QUALITY_CAMERA_CONFIG = {
    "pixel_height": 1080,
    "pixel_width": 1920,
    "frame_rate": 60,
}

MEDIUM_QUALITY_CAMERA_CONFIG = {
    "pixel_height": 720,
    "pixel_width": 1280,
    "frame_rate": 30,
}

LOW_QUALITY_CAMERA_CONFIG = {
    "pixel_height": 480,
    "pixel_width": 854,
    "frame_rate": 15,
}

DEFAULT_PIXEL_HEIGHT = PRODUCTION_QUALITY_CAMERA_CONFIG["pixel_height"]
DEFAULT_PIXEL_WIDTH = PRODUCTION_QUALITY_CAMERA_CONFIG["pixel_width"]
DEFAULT_FRAME_RATE = 60

DEFAULT_POINT_DENSITY_2D = 25
DEFAULT_POINT_DENSITY_1D = 250

DEFAULT_STROKE_WIDTH = 4

FRAME_HEIGHT = 8.0
FRAME_WIDTH = FRAME_HEIGHT * DEFAULT_PIXEL_WIDTH / DEFAULT_PIXEL_HEIGHT
FRAME_Y_RADIUS = FRAME_HEIGHT / 2
FRAME_X_RADIUS = FRAME_WIDTH / 2

SMALL_BUFF = 0.1
MED_SMALL_BUFF = 0.25
MED_LARGE_BUFF = 0.5
LARGE_BUFF = 1

DEFAULT_MOBJECT_TO_EDGE_BUFFER = MED_LARGE_BUFF
DEFAULT_MOBJECT_TO_MOBJECT_BUFFER = MED_SMALL_BUFF


# All in seconds
DEFAULT_POINTWISE_FUNCTION_RUN_TIME = 3.0
DEFAULT_WAIT_TIME = 1.0


ORIGIN = np.array((0., 0., 0.))
UP = np.array((0., 1., 0.))
DOWN = np.array((0., -1., 0.))
RIGHT = np.array((1., 0., 0.))
LEFT = np.array((-1., 0., 0.))
IN = np.array((0., 0., -1.))
OUT = np.array((0., 0., 1.))
X_AXIS = np.array((1., 0., 0.))
Y_AXIS = np.array((0., 1., 0.))
Z_AXIS = np.array((0., 0., 1.))

# Useful abbreviations for diagonals
UL = UP + LEFT
UR = UP + RIGHT
DL = DOWN + LEFT
DR = DOWN + RIGHT

TOP = FRAME_Y_RADIUS * UP
BOTTOM = FRAME_Y_RADIUS * DOWN
LEFT_SIDE = FRAME_X_RADIUS * LEFT
RIGHT_SIDE = FRAME_X_RADIUS * RIGHT

PI = np.pi
TAU = 2 * PI
DEGREES = TAU / 360

FFMPEG_BIN = "ffmpeg"

# Colors
COLOR_MAP = {
    "DARK_BLUE": "#236B8E",
    "DARK_BROWN": "#8B4513",
    "LIGHT_BROWN": "#CD853F",
    "BLUE_E": "#1C758A",
    "BLUE_D": "#29ABCA",
    "BLUE_C": "#58C4DD",
    "BLUE_B": "#9CDCEB",
    "BLUE_A": "#C7E9F1",
    "TEAL_E": "#49A88F",
    "TEAL_D": "#55C1A7",
    "TEAL_C": "#5CD0B3",
    "TEAL_B": "#76DDC0",
    "TEAL_A": "#ACEAD7",
    "GREEN_E": "#699C52",
    "GREEN_D": "#77B05D",
    "GREEN_C": "#83C167",
    "GREEN_B": "#A6CF8C",
    "GREEN_A": "#C9E2AE",
    "YELLOW_E": "#E8C11C",
    "YELLOW_D": "#F4D345",
    "YELLOW_C": "#FFFF00",
    "YELLOW_B": "#FFEA94",
    "YELLOW_A": "#FFF1B6",
    "GOLD_E": "#C78D46",
    "GOLD_D": "#E1A158",
    "GOLD_C": "#F0AC5F",
    "GOLD_B": "#F9B775",
    "GOLD_A": "#F7C797",
    "RED_E": "#CF5044",
    "RED_D": "#E65A4C",
    "RED_C": "#FC6255",
    "RED_B": "#FF8080",
    "RED_A": "#F7A1A3",
    "MAROON_E": "#94424F",
    "MAROON_D": "#A24D61",
    "MAROON_C": "#C55F73",
    "MAROON_B": "#EC92AB",
    "MAROON_A": "#ECABC1",
    "PURPLE_E": "#644172",
    "PURPLE_D": "#715582",
    "PURPLE_C": "#9A72AC",
    "PURPLE_B": "#B189C6",
    "PURPLE_A": "#CAA3E8",
    "WHITE": "#FFFFFF",
    "BLACK": "#000000",
    "LIGHT_GRAY": "#BBBBBB",
    "LIGHT_GREY": "#BBBBBB",
    "GRAY": "#888888",
    "GREY": "#888888",
    "DARK_GREY": "#444444",
    "DARK_GRAY": "#444444",
    "DARKER_GREY": "#222222",
    "DARKER_GRAY": "#222222",
    "GREY_BROWN": "#736357",
    "PINK": "#D147BD",
    "LIGHT_PINK": "#DC75CD",
    "GREEN_SCREEN": "#00FF00",
    "ORANGE": "#FF862F",
}
PALETTE = list(COLOR_MAP.values())
locals().update(COLOR_MAP)
for name in [s for s in list(COLOR_MAP.keys()) if s.endswith("_C")]:
    locals()[name.replace("_C", "")] = locals()[name]

# Streaming related configuration
LIVE_STREAM_NAME = "LiveStream"
TWITCH_STREAM_KEY = "YOUR_STREAM_KEY"
STREAMING_PROTOCOL = "tcp"
STREAMING_IP = "127.0.0.1"
STREAMING_PORT = "2000"
STREAMING_CLIENT = "ffplay"
STREAMING_URL = f"{STREAMING_PROTOCOL}://{STREAMING_IP}:{STREAMING_PORT}?listen"
STREAMING_CONSOLE_BANNER = """
Manim is now running in streaming mode. Stream animations by passing
them to manim.play(), e.g.
>>> c = Circle()
>>> manim.play(ShowCreation(c))
"""
code_languages_list = {"abap": "abap", "as": "as", "as3": "as3", "ada": "ada", "antlr": "antlr",
                       "antlr_as": "antlr-as",
                       "antlr_csharp": "antlr-csharp", "antlr_cpp": "antlr-cpp", "antlr_java": "antlr-java",
                       "antlr_objc": "antlr-objc", "antlr_perl": "antlr-perl", "antlr_python": "antlr-python",
                       "antlr_ruby": "antlr-ruby", "apacheconf": "apacheconf", "applescript": "applescript",
                       "aspectj": "aspectj",
                       "aspx_cs": "aspx-cs", "aspx_vb": "aspx-vb", "asy": "asy", "ahk": "ahk", "autoit": "autoit",
                       "awk": "awk",
                       "basemake": "basemake", "bash": "bash", "console": "console", "bat": "bat",
                       "bbcode": "bbcode",
                       "befunge": "befunge", "blitzmax": "blitzmax", "boo": "boo", "brainfuck": "brainfuck",
                       "bro": "bro",
                       "bugs": "bugs", "c": "c", "csharp": "csharp", "cpp": "cpp", "c_objdump": "c-objdump",
                       "ca65": "ca65",
                       "cbmbas": "cbmbas", "ceylon": "ceylon", "cfengine3": "cfengine3", "cfs": "cfs",
                       "cheetah": "cheetah",
                       "clojure": "clojure", "cmake": "cmake", "cobol": "cobol", "cobolfree": "cobolfree",
                       "coffee_script": "coffee-script", "cfm": "cfm", "common_lisp": "common-lisp", "coq": "coq",
                       "cpp_objdump": "cpp-objdump", "croc": "croc", "css": "css", "css_django": "css+django",
                       "css_genshitext": "css+genshitext", "css_lasso": "css+lasso", "css_mako": "css+mako",
                       "css_myghty": "css+myghty", "css_php": "css+php", "css_erb": "css+erb",
                       "css_smarty": "css+smarty",
                       "cuda": "cuda", "cython": "cython", "d": "d", "d_objdump": "d-objdump", "dpatch": "dpatch",
                       "dart": "dart",
                       "control": "control", "sourceslist": "sourceslist", "delphi": "delphi", "dg": "dg",
                       "diff": "diff",
                       "django": "django", "dtd": "dtd", "duel": "duel", "dylan": "dylan",
                       "dylan_console": "dylan-console",
                       "dylan_lid": "dylan-lid", "ec": "ec", "ecl": "ecl", "elixir": "elixir", "iex": "iex",
                       "ragel_em": "ragel-em",
                       "erb": "erb", "erlang": "erlang", "erl": "erl", "evoque": "evoque", "factor": "factor",
                       "fancy": "fancy",
                       "fan": "fan", "felix": "felix", "fortran": "fortran", "Clipper": "Clipper",
                       "fsharp": "fsharp", "gas": "gas",
                       "genshi": "genshi", "genshitext": "genshitext", "pot": "pot", "Cucumber": "Cucumber",
                       "glsl": "glsl",
                       "gnuplot": "gnuplot", "go": "go", "gooddata_cl": "gooddata-cl", "gosu": "gosu", "gst": "gst",
                       "groff": "groff",
                       "groovy": "groovy", "haml": "haml", "haskell": "haskell", "hx": "hx", "html": "html",
                       "html_cheetah": "html+cheetah", "html_django": "html+django", "html_evoque": "html+evoque",
                       "html_genshi": "html+genshi", "html_lasso": "html+lasso", "html_mako": "html+mako",
                       "html_myghty": "html+myghty", "html_php": "html+php", "html_smarty": "html+smarty",
                       "html_velocity": "html+velocity", "http": "http", "haxeml": "haxeml", "hybris": "hybris",
                       "idl": "idl",
                       "ini": "ini", "io": "io", "ioke": "ioke", "irc": "irc", "jade": "jade", "jags": "jags",
                       "java": "java",
                       "jsp": "jsp", "js": "js", "js_cheetah": "js+cheetah", "js_django": "js+django",
                       "js_genshitext": "js+genshitext", "js_lasso": "js+lasso", "js_mako": "js+mako",
                       "js_myghty": "js+myghty",
                       "js_php": "js+php", "js_erb": "js+erb", "js_smarty": "js+smarty", "json": "json",
                       "julia": "julia",
                       "jlcon": "jlcon", "kconfig": "kconfig", "koka": "koka", "kotlin": "kotlin", "lasso": "lasso",
                       "lighty": "lighty", "lhs": "lhs", "live_script": "live-script", "llvm": "llvm",
                       "logos": "logos",
                       "logtalk": "logtalk", "lua": "lua", "make": "make", "mako": "mako", "maql": "maql",
                       "mason": "mason",
                       "matlab": "matlab", "matlabsession": "matlabsession", "minid": "minid",
                       "modelica": "modelica",
                       "modula2": "modula2", "trac_wiki": "trac-wiki", "monkey": "monkey", "moocode": "moocode",
                       "moon": "moon",
                       "mscgen": "mscgen", "mupad": "mupad", "mxml": "mxml", "myghty": "myghty", "mysql": "mysql",
                       "nasm": "nasm",
                       "nemerle": "nemerle", "newlisp": "newlisp", "newspeak": "newspeak", "nginx": "nginx",
                       "nimrod": "nimrod",
                       "nsis": "nsis", "numpy": "numpy", "objdump": "objdump", "objective_c": "objective-c",
                       "objective_c_+": "objective-c++", "objective_j": "objective-j", "ocaml": "ocaml",
                       "octave": "octave",
                       "ooc": "ooc", "opa": "opa", "openedge": "openedge", "perl": "perl", "php": "php",
                       "plpgsql": "plpgsql",
                       "psql": "psql", "postgresql": "postgresql", "postscript": "postscript", "pov": "pov",
                       "powershell": "powershell", "prolog": "prolog", "properties": "properties",
                       "protobuf": "protobuf",
                       "puppet": "puppet", "pypylog": "pypylog", "python": "python", "python3": "python3",
                       "py3tb": "py3tb",
                       "pycon": "pycon", "pytb": "pytb", "qml": "qml", "racket": "racket", "ragel": "ragel",
                       "ragel_c": "ragel-c",
                       "ragel_cpp": "ragel-cpp", "ragel_d": "ragel-d", "ragel_java": "ragel-java",
                       "ragel_objc": "ragel-objc",
                       "ragel_ruby": "ragel-ruby", "raw": "raw", "rconsole": "rconsole", "rd": "rd",
                       "rebol": "rebol",
                       "redcode": "redcode", "registry": "registry", "rst": "rst", "rhtml": "rhtml",
                       "RobotFramework": "RobotFramework", "spec": "spec", "rb": "rb", "rbcon": "rbcon",
                       "rust": "rust",
                       "splus": "splus", "sass": "sass", "scala": "scala", "ssp": "ssp", "scaml": "scaml",
                       "scheme": "scheme",
                       "scilab": "scilab", "scss": "scss", "shell_session": "shell-session", "smali": "smali",
                       "smalltalk": "smalltalk", "smarty": "smarty", "snobol": "snobol", "sp": "sp", "sql": "sql",
                       "sqlite3": "sqlite3", "squidconf": "squidconf", "stan": "stan", "sml": "sml",
                       "systemverilog": "systemverilog",
                       "tcl": "tcl", "tcsh": "tcsh", "tea": "tea", "tex": "tex", "text": "text",
                       "treetop": "treetop", "ts": "ts",
                       "urbiscript": "urbiscript", "vala": "vala", "vb.net": "vb.net", "velocity": "velocity",
                       "verilog": "verilog",
                       "vgl": "vgl", "vhdl": "vhdl", "vim": "vim", "xml": "xml", "xml_cheetah": "xml+cheetah",
                       "xml_django": "xml+django", "xml_evoque": "xml+evoque", "xml_lasso": "xml+lasso",
                       "xml_mako": "xml+mako",
                       "xml_myghty": "xml+myghty", "xml_php": "xml+php", "xml_erb": "xml+erb",
                       "xml_smarty": "xml+smarty",
                       "xml_velocity": "xml+velocity", "xquery": "xquery", "xslt": "xslt", "xtend": "xtend",
                       "yaml": "yaml"}

code_styles_list = {0: "autumn", 1: "borland", 2: "bw", 3: "colorful", 4: "default", 5: "emacs",
                    6: "friendly", 7: "fruity", 8: "manni", 9: "monokai", 10: "murphy", 11: "native",
                    12: "pastie", 13: "perldoc", 14: "rrt", 15: "tango", 16: "trac", 17: "vim", 18: "vs"}



#!/usr/bin/env python
import manimlib.config
import manimlib.constants
import manimlib.extract_scene


def main():
    args = manimlib.config.parse_cli()
    config = manimlib.config.get_configuration(args)
    manimlib.constants.initialize_directories(config)
    manimlib.extract_scene.main(config)



import argparse
import colour
import importlib.util
import os
import sys
import types

import manimlib.constants


def parse_cli():
    try:
        parser = argparse.ArgumentParser()
        parser.add_argument(
            "file",
            help="path to file holding the python code for the scene",
        )
        parser.add_argument(
            "scene_names",
            nargs="*",
            help="Name of the Scene class you want to see",
        )
        parser.add_argument(
            "-p", "--preview",
            action="store_true",
            help="Automatically open the saved file once its done",
        ),
        parser.add_argument(
            "-w", "--write_to_movie",
            action="store_true",
            help="Render the scene as a movie file",
        ),
        parser.add_argument(
            "-s", "--save_last_frame",
            action="store_true",
            help="Save the last frame",
        ),
        parser.add_argument(
            "-l", "--low_quality",
            action="store_true",
            help="Render at a low quality (for faster rendering)",
        ),
        parser.add_argument(
            "-m", "--medium_quality",
            action="store_true",
            help="Render at a medium quality",
        ),
        parser.add_argument(
            "--high_quality",
            action="store_true",
            help="Render at a high quality",
        ),
        parser.add_argument(
            "-g", "--save_pngs",
            action="store_true",
            help="Save each frame as a png",
        ),
        parser.add_argument(
            "-i", "--save_as_gif",
            action="store_true",
            help="Save the video as gif",
        ),
        parser.add_argument(
            "-f", "--show_file_in_finder",
            action="store_true",
            help="Show the output file in finder",
        ),
        parser.add_argument(
            "-t", "--transparent",
            action="store_true",
            help="Render to a movie file with an alpha channel",
        ),
        parser.add_argument(
            "-q", "--quiet",
            action="store_true",
            help="",
        ),
        parser.add_argument(
            "-a", "--write_all",
            action="store_true",
            help="Write all the scenes from a file",
        ),
        parser.add_argument(
            "-o", "--file_name",
            help="Specify the name of the output file, if"
                 "it should be different from the scene class name",
        )
        parser.add_argument(
            "-n", "--start_at_animation_number",
            help="Start rendering not from the first animation, but"
                 "from another, specified by its index.  If you pass"
                 "in two comma separated values, e.g. \"3,6\", it will end"
                 "the rendering at the second value",
        )
        parser.add_argument(
            "-r", "--resolution",
            help="Resolution, passed as \"height,width\"",
        )
        parser.add_argument(
            "-c", "--color",
            help="Background color",
        )
        parser.add_argument(
            "--sound",
            action="store_true",
            help="Play a success/failure sound",
        )
        parser.add_argument(
            "--leave_progress_bars",
            action="store_true",
            help="Leave progress bars displayed in terminal",
        )
        parser.add_argument(
            "--media_dir",
            help="directory to write media",
        )
        video_group = parser.add_mutually_exclusive_group()
        video_group.add_argument(
            "--video_dir",
            help="directory to write file tree for video",
        )
        video_group.add_argument(
            "--video_output_dir",
            help="directory to write video",
        )
        parser.add_argument(
            "--tex_dir",
            help="directory to write tex",
        )
        return parser.parse_args()
    except argparse.ArgumentError as err:
        print(str(err))
        sys.exit(2)


def get_module(file_name):
    if file_name == "-":
        module = types.ModuleType("input_scenes")
        code = "from manimlib.imports import *\n\n" + sys.stdin.read()
        try:
            exec(code, module.__dict__)
            return module
        except Exception as e:
            print(f"Failed to render scene: {str(e)}")
            sys.exit(2)
    else:
        module_name = file_name.replace(os.sep, ".").replace(".py", "")
        spec = importlib.util.spec_from_file_location(module_name, file_name)
        module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(module)
        return module


def get_configuration(args):
    module = get_module(args.file)
    file_writer_config = {
        # By default, write to file
        "write_to_movie": args.write_to_movie or not args.save_last_frame,
        "save_last_frame": args.save_last_frame,
        "save_pngs": args.save_pngs,
        "save_as_gif": args.save_as_gif,
        # If -t is passed in (for transparent), this will be RGBA
        "png_mode": "RGBA" if args.transparent else "RGB",
        "movie_file_extension": ".mov" if args.transparent else ".mp4",
        "file_name": args.file_name,
        "input_file_path": args.file,
    }
    if hasattr(module, "OUTPUT_DIRECTORY"):
        file_writer_config["output_directory"] = module.OUTPUT_DIRECTORY
    config = {
        "module": module,
        "scene_names": args.scene_names,
        "open_video_upon_completion": args.preview,
        "show_file_in_finder": args.show_file_in_finder,
        "file_writer_config": file_writer_config,
        "quiet": args.quiet or args.write_all,
        "ignore_waits": args.preview,
        "write_all": args.write_all,
        "start_at_animation_number": args.start_at_animation_number,
        "end_at_animation_number": None,
        "sound": args.sound,
        "leave_progress_bars": args.leave_progress_bars,
        "media_dir": args.media_dir,
        "video_dir": args.video_dir,
        "video_output_dir": args.video_output_dir,
        "tex_dir": args.tex_dir,
    }

    # Camera configuration
    config["camera_config"] = get_camera_configuration(args)

    # Arguments related to skipping
    stan = config["start_at_animation_number"]
    if stan is not None:
        if "," in stan:
            start, end = stan.split(",")
            config["start_at_animation_number"] = int(start)
            config["end_at_animation_number"] = int(end)
        else:
            config["start_at_animation_number"] = int(stan)

    config["skip_animations"] = any([
        file_writer_config["save_last_frame"],
        config["start_at_animation_number"],
    ])
    return config


def get_camera_configuration(args):
    camera_config = {}
    if args.low_quality:
        camera_config.update(manimlib.constants.LOW_QUALITY_CAMERA_CONFIG)
    elif args.medium_quality:
        camera_config.update(manimlib.constants.MEDIUM_QUALITY_CAMERA_CONFIG)
    elif args.high_quality:
        camera_config.update(manimlib.constants.HIGH_QUALITY_CAMERA_CONFIG)
    else:
        camera_config.update(manimlib.constants.PRODUCTION_QUALITY_CAMERA_CONFIG)

    # If the resolution was passed in via -r
    if args.resolution:
        if "," in args.resolution:
            height_str, width_str = args.resolution.split(",")
            height = int(height_str)
            width = int(width_str)
        else:
            height = int(args.resolution)
            width = int(16 * height / 9)
        camera_config.update({
            "pixel_height": height,
            "pixel_width": width,
        })

    if args.color:
        try:
            camera_config["background_color"] = colour.Color(args.color)
        except AttributeError as err:
            print("Please use a valid color")
            print(err)
            sys.exit(2)

    # If rendering a transparent image/move, make sure the
    # scene has a background opacity of 0
    if args.transparent:
        camera_config["background_opacity"] = 0

    return camera_config



import warnings
import numpy as np

from manimlib.constants import *
from manimlib.mobject.geometry import *
from manimlib.mobject.types.vectorized_mobject import VMobject

#CONSTANTS
LOGIC_WIDTH = 2.0;
LOGIC_HEIGHT = 1.0;


class LogicGate(VMobject):
	CONFIG = {
	"inputA_loc" : 0,
	"inputB_loc" : 0,
	}

	def __init__(self,**kwargs):
		VMobject.__init__(self,**kwargs)


	def set_inputs(self,input1,input2):
		input1_coord = input1.get_points()[0]
		input2_coord = input2.get_points()[0]


        #Getting point locations
		idx = 0
		for coord in self.get_points():
			if coord[0] == input1_coord[0] and coord[1] == input1_coord[1] and coord[2] == input1_coord[2]:
				self.inputA_loc = idx

			if coord[0] == input2_coord[0] and coord[1] == input2_coord[1] and coord[2] == input2_coord[2]:
				self.inputB_loc = idx

			idx += 1







	def get_inputA(self):
		return self.get_points()[self.inputA_loc]

	def get_inputB(self):
		return self.get_points()[self.inputB_loc]

	def get_output(self):
		return self.get_points()[-1]



class AND(LogicGate):


	#input1 = Line()
	#input2 = Line()
	def generate_points(self):

		input1 = Line((-0.5,0.25,0),(-0.1,0.25,0))
		input2 = Line((-0.5,-0.25,0),(-0.1,-0.25,0))
		back = Line((-0.1,-0.5,0),(-0.1,0.5,0))
		top = Line((-0.1,0.5,0),(0.5,0.5,0))
		bot = Line((0.5,-0.5,0),(-0.1,-0.5,0))
		front = Arc(PI/2,-PI,radius=0.5)
		front.shift(RIGHT*0.5)
		output = Line((1,0,0),(1.5,0,0))

		self.append_points(input1.get_points())
		self.append_points(input2.get_points())
		self.append_points(back.get_points())
		self.append_points(top.get_points())
		self.append_points(front.get_points())
		self.append_points(bot.get_points())
		self.append_points(output.get_points())

		self.set_inputs(input1,input2)
		self.shift(LEFT*0.5)

		








class NAND(LogicGate):

	def generate_points(self):


		input1 = Line((-0.5,0.25,0),(-0.1,0.25,0))
		input2 = Line((-0.5,-0.25,0),(-0.1,-0.25,0))
		back = Line((-0.1,-0.5,0),(-0.1,0.5,0))
		top = Line((-0.1,0.5,0),(0.5,0.5,0))
		bot = Line((0.5,-0.5,0),(-0.1,-0.5,0))
		front = Arc(PI/2,-PI,radius=0.5)
		front.shift(RIGHT*0.5)
		output = Line((1.125,0,0),(1.5,0,0))
		
		neg = Circle(radius=0.0625)
		neg.shift(RIGHT*(1 + 0.0625))


		self.append_points(input1.get_points())
		self.append_points(input2.get_points())
		self.append_points(back.get_points())
		self.append_points(top.get_points())
		self.append_points(front.get_points())
		self.append_points(bot.get_points())
		self.append_points(neg.get_points())
		self.append_points(output.get_points())

		self.set_inputs(input1,input2)
		self.shift(LEFT*0.5)

		


# Drawing the OR,NOR,XOR & XNOR shapes were tricky
# used plenty of useful online sources such as

# Drawing perfect parabolas with quadratic bezier: https://www.math.fsu.edu/~rabert/TeX/parabola/parabola.html
# Aproximate quadratic with cubic bezier: https://stackoverflow.com/questions/3162645/convert-a-quadratic-bezier-to-a-cubic-one
# Drawing hald parabola: https://tex.stackexchange.com/questions/285255/drawing-half-a-parabola-using-pstricks-in-latex
class OR(LogicGate):

	def generate_points(self):

		#QP0 = -0.125,0.5
		#QP1 = 0.125,0
		#QP2 = -0.125,-0.5


		# QP0 = (-0.25,0.5,0)
		# QP1 = (0.25,0,0)
		# QP2 = (-0.25,-0.5,0)
		
		# CP0 = QP0
		# CP3 = QP2

		#CP1 = QP0 + 2/3*(QP1-QP0)
		#CP2 = QP2 + 2/3*(QP1-QP2)

		#CP1 = (-0.25,0.5,0) + 2/3*((0.25,0,0) - (-0.25,0.5,0)) = (0.083,0.0.166,0)
		#CP2 = (-0.25,-0.5,0) + 2/3* ( (0.25,0,0) -  (-0.25,-0.5,0) ) = (0.083,-0.166,0)


		back = [(-0.25,0.5,0),(0.083,0.166,0),(0.083,-0.166,0),(-0.25,-0.5,0)]


		# QP0 = (−0.45710678,0,0)
		# QP1 = (0.25,1,0)
		# QP2 = (0.95710678,0,0)


		# CP0 = QP0
		# CP1 = (0.01429774,2/3,0)
		# CP2 = (0.48570226,2/3,0)
		# CP3 = QP2

		input1 = Line((-0.5,0.4,0),(-0.16,0.4,0))
		input2 = Line((-0.5,-0.4,0),(-0.16,-0.4,0))
		output = Line((0.95710678,0,0),(1.5,0,0))
		verts2 = [(-0.45710678,0,0),(0.01429774,2/3,0),(0.48570226,2/3,0),(0.95710678,0,0)]
		topCurve = [(0.25,0.5,0),(0.48570226,0.5,0),(0.72140452,1/3,0),(0.95710678,0,0)]
		botCurve = [(0.25,-0.5,0),(0.48570226,-0.5,0),(0.72140452,-1/3,0),(0.95710678,0,0)]


		#topCurvePoints = topCurve.get_points()
		#print(topCurvePoints)
		top = Line((-0.25,0.5,0),(0.255,0.5,0))
		bot = Line((0.25,-0.5,0),(-0.25,-0.5,0))
		#self.add_cubic_bezier_curve_to(self,vert1[0],vert1[1])
		
		self.append_points(input1.get_points())
		self.append_points(input2.get_points())
		self.append_points(topCurve)
		self.append_points(botCurve[::-1])
		self.append_points(bot.get_points())
		self.append_points(back[::-1])
		self.append_points(top.get_points())
		self.append_points(output.get_points())

		self.set_inputs(input1,input2)
		self.shift(LEFT*0.5)

		

class NOR(LogicGate):

	def generate_points(self):
		back = [(-0.25,0.5,0),(0.083,0.166,0),(0.083,-0.166,0),(-0.25,-0.5,0)]

		input1 = Line((-0.5,0.4,0),(-0.16,0.4,0))
		input2 = Line((-0.5,-0.4,0),(-0.16,-0.4,0))
		neg = Circle(radius=0.0625)
		neg.shift( (0.95710678+0.0625)*RIGHT)

		output = Line((0.95710678+0.125,0,0),(1.5,0,0))
		verts2 = [(-0.45710678,0,0),(0.01429774,2/3,0),(0.48570226,2/3,0),(0.95710678,0,0)]
		topCurve = [(0.25,0.5,0),(0.48570226,0.5,0),(0.72140452,1/3,0),(0.95710678,0,0)]
		botCurve = [(0.25,-0.5,0),(0.48570226,-0.5,0),(0.72140452,-1/3,0),(0.95710678,0,0)]

		top = Line((-0.25,0.5,0),(0.255,0.5,0))
		bot = Line((0.25,-0.5,0),(-0.25,-0.5,0))

		self.append_points(input1.get_points())
		self.append_points(input2.get_points())
		self.append_points(topCurve)
		self.append_points(botCurve[::-1])
		self.append_points(bot.get_points())
		self.append_points(back[::-1])
		self.append_points(top.get_points())

		self.append_points(neg.get_points())

		self.append_points(output.get_points())

		self.set_inputs(input1,input2)
		self.shift(LEFT*0.5)

		


class XOR(LogicGate):
	def generate_points(self):

		back1= [(-0.25-0.1,0.5,0),(0.083-0.1,0.166,0),(0.083-0.1,-0.166,0),(-0.25-0.1,-0.5,0)] 
		back2= [(-0.25,0.5,0),(0.083,0.166,0),(0.083,-0.166,0),(-0.25,-0.5,0)]

		input1 = Line((-0.5,0.4,0),(-0.16-0.1,0.4,0))
		input2 = Line((-0.5,-0.4,0),(-0.16-0.1,-0.4,0))

		output = Line((0.95710678,0,0),(1.5,0,0))
		verts2 = [(-0.45710678,0,0),(0.01429774,2/3,0),(0.48570226,2/3,0),(0.95710678,0,0)]
		topCurve = [(0.25,0.5,0),(0.48570226,0.5,0),(0.72140452,1/3,0),(0.95710678,0,0)]
		botCurve = [(0.25,-0.5,0),(0.48570226,-0.5,0),(0.72140452,-1/3,0),(0.95710678,0,0)]

		top = Line((-0.25,0.5,0),(0.255,0.5,0))
		bot = Line((0.25,-0.5,0),(-0.25,-0.5,0))

		self.append_points(input1.get_points())
		self.append_points(input2.get_points())
		self.append_points(back1)

		
		
		self.append_points(topCurve)
		self.append_points(botCurve[::-1])
		self.append_points(bot.get_points())
		self.append_points(back2[::-1])
		self.append_points(top.get_points())

		self.append_points(output.get_points())	

		self.set_inputs(input1,input2)
		self.shift(LEFT*0.5)

		



class XNOR(LogicGate):
	def generate_points(self):

		back1= [(-0.25-0.1,0.5,0),(0.083-0.1,0.166,0),(0.083-0.1,-0.166,0),(-0.25-0.1,-0.5,0)] 
		back2= [(-0.25,0.5,0),(0.083,0.166,0),(0.083,-0.166,0),(-0.25,-0.5,0)]

		input1 = Line((-0.5,0.4,0),(-0.16-0.1,0.4,0))
		input2 = Line((-0.5,-0.4,0),(-0.16-0.1,-0.4,0))
		neg = Circle(radius=0.0625)
		neg.shift( (0.95710678+0.0625)*RIGHT)

		output = Line((0.95710678+0.125,0,0),(1.5,0,0))
		verts2 = [(-0.45710678,0,0),(0.01429774,2/3,0),(0.48570226,2/3,0),(0.95710678,0,0)]
		topCurve = [(0.25,0.5,0),(0.48570226,0.5,0),(0.72140452,1/3,0),(0.95710678,0,0)]
		botCurve = [(0.25,-0.5,0),(0.48570226,-0.5,0),(0.72140452,-1/3,0),(0.95710678,0,0)]

		top = Line((-0.25,0.5,0),(0.255,0.5,0))
		bot = Line((0.25,-0.5,0),(-0.25,-0.5,0))

		self.append_points(input1.get_points())
		self.append_points(input2.get_points())
		self.append_points(back1)

		self.append_points(topCurve)
		self.append_points(botCurve[::-1])
		self.append_points(bot.get_points())
		self.append_points(back2[::-1])
		self.append_points(top.get_points())

		self.append_points(neg.get_points())

		self.append_points(output.get_points())

		self.set_inputs(input1,input2)
		self.shift(LEFT*0.5)	

		


class Buffer(LogicGate):

	def generate_points(self):

		verts = [(-0.5,0,0),(0,0,0),(0,-0.5,0),(np.sin(PI/3),0,0),(0,0.5,0)]
		output = Line((np.sin(PI/3),0,0),(1.5,0,0))

		self.set_points_as_corners( [*verts,verts[1]] )
		self.append_points(output.get_points())

		self.inputA_loc = 0		
		self.shift(LEFT*0.5)

		


class NOT(LogicGate):

	def generate_points(self):

		verts = [(-0.5,0,0),(0,0,0),(0,-0.5,0),(np.sin(PI/3),0,0),(0,0.5,0)]
		output = Line((np.sin(PI/3)+0.125,0,0),(1.5,0,0))
		neg = Circle(radius=0.0625)
		neg.shift( (np.sin(PI/3)+0.0625)*RIGHT)

		self.set_points_as_corners( [*verts,verts[1]] )
		self.append_points(neg.get_points())
		self.append_points(output.get_points())

		self.inputA_loc = 0
		self.shift(LEFT*0.5)

		

		


import warnings
import numpy as np

from manimlib.constants import *
from manimlib.mobject.geometry import *
from manimlib.mobject.types.vectorized_mobject import VMobject


class WeirdShape(VMobject):
    def __init__(self, points, **kwargs):
        VMobject.__init__(self, **kwargs)
        self.set_points_as_corners(points)

class Circuit(VMobject):

    def __init__(self,**kwargs):
        self.ps = []
        self.bs = []
        VMobject.__init__(self,**kwargs)


    def connect(self,point1,point2,pin_top=False):
        if(pin_top):
            self.ps.append([point1,(point2[0],point1[1],0),point2])
        else:
            self.ps.append([point1,(point1[0],point2[1],0),point2])

    def connect_right_to_left(self,point1,point2):
        target_point = (point2[0],point1[1]-1.5,0)

        self.connect(point1,target_point)
        self.connect(target_point,point2)

    def render(self):
        for p in self.ps:
            self.append_points(WeirdShape(p).get_points());


class CircuitComponent(VMobject):


    def get_left(self):
        return self.get_points()[0];

    def get_right(self): 
        return self.get_points()[len(self.get_points()) - 1];



class Resistor(CircuitComponent):
    CONFIG = {
        "color" : WHITE,
        "points": [(-0.5,0,0),(-0.3125,0,0),(-0.25,-0.25,0),(-0.125,0.25,0),(0,-0.25,0),(0.125,0.25,0),(0.25,-0.25,0),(0.3125,0,0),(0.5,0,0)]
    }
    def __init__(self, **kwargs):
        VMobject.__init__(self, **kwargs)
        self.set_points_as_corners(self.CONFIG["points"])   

class Capacitor(CircuitComponent):
    CONFIG = {
        "is_polar" : False
    }
    pos_side = 0;
    neg_side = 0;

    def __init__(self,**kwargs):
        VMobject.__init__(self,**kwargs)

    def generate_points(self):

        pos = Line((-0.5,0,0),(-0.125,0,0))
        if (self.is_polar):
            wall1 = CubicBezier( [(-0.125,0.5,0),(0.083/2,0.166,0),(0.083/2,-0.166,0),(-0.125,-0.5,0)])
            wall1.shift(LEFT*0.125)
        else:
            wall1 = Line((-0.125,0.5,0),(-0.125,-0.5,0))

        wall2 = Line((0.125,0.5,0),(0.125,-0.5,0))
        neg = Line((0.125,0,0),(0.5,0,0))
        self.append_points(pos.get_points())
        self.append_points(wall1.get_points())
        self.append_points(wall2.get_points())
        self.append_points(neg.get_points())  



class Battery(CircuitComponent):
    CONFIG = {
        "color" : WHITE
    }

    def generate_points(self):
        neg = Line((-0.5,0,0),(-0.125,0,0))
        wall1 = Line((-0.125,0.25,0),(-0.125,-0.25,0))
        wall2 = Line((0.125,0.5,0),(0.125,-0.5,0))
        pos = Line((0.125,0,0),(0.5,0,0))
        self.append_points(neg.get_points())
        self.append_points(wall1.get_points())
        self.append_points(wall2.get_points())
        self.append_points(pos.get_points())    

class Inductor(CircuitComponent):
    def generate_points(self):
        line1 = Line((-0.5,0,0),(-0.375,0,0))
        arc1 = Arc(TAU/2, -TAU/2,radius=0.125)
        arc1.shift(0.25*LEFT)
        arc2 = Arc(TAU/2, -TAU/2,radius=0.125)
        arc3 = Arc(TAU/2, -TAU/2,radius=0.125)
        arc3.shift(0.25*RIGHT)
        line2 = Line((0.375,0,0),(0.5,0,0))

        self.append_points(line1.get_points())
        self.append_points(arc1.get_points())
        self.append_points(arc2.get_points())
        self.append_points(arc3.get_points())
        self.append_points(line2.get_points())

# Diode Enums
DIODE_DEFAULT = 0
DIODE_ZENER = 1;
DIODE_SCHOTTKY = 2;
class Diode(CircuitComponent):
    CONFIG = {
    "diode_type" : DIODE_DEFAULT
    }

    def __init__(self,**kwargs):
        VMobject.__init__(self,**kwargs)

    def generate_points(self):
        line1 = Line((-0.5,0,0),(-0.25,0,0))
        tri = WeirdShape([(-0.25,0.25,0),(-0.25,-0.25,0),(np.sin(PI/3)*0.5-0.25,0,0),(-0.25,0.25,0)])

        if(self.diode_type == DIODE_ZENER):
            l1 = Line((np.sin(PI/3)*0.5-0.1,0.3,0),(np.sin(PI/3)*0.5-0.25,0.3,0))
            l2 = Line((np.sin(PI/3)*0.5-0.25,0.3,0),(np.sin(PI/3)*0.5-0.25,-0.3,0))
            l3 = Line((np.sin(PI/3)*0.5-0.25,-0.3,0),(np.sin(PI/3)*0.5-0.4,-0.3,0))

            front = [*l1.get_points(),*l2.get_points(),*l3.get_points()]
        elif(self.diode_type == DIODE_SCHOTTKY):
            l0 = Line((np.sin(PI/3)*0.5-0.1,0.25,0),(np.sin(PI/3)*0.5-0.1,0.3,0))
            l1 = Line((np.sin(PI/3)*0.5-0.1,0.3,0),(np.sin(PI/3)*0.5-0.25,0.3,0))
            l2 = Line((np.sin(PI/3)*0.5-0.25,0.3,0),(np.sin(PI/3)*0.5-0.25,-0.3,0))
            l3 = Line((np.sin(PI/3)*0.5-0.25,-0.3,0),(np.sin(PI/3)*0.5-0.4,-0.3,0))
            l4 = Line((np.sin(PI/3)*0.5-0.4,-0.3,0),(np.sin(PI/3)*0.5-0.4,-0.25,0))


            front = [*l0.get_points(),*l1.get_points(),*l2.get_points(),*l3.get_points(),*l4.get_points()]
        else:
            l1 = Line((np.sin(PI/3)*0.5-0.25,0.3,0),(np.sin(PI/3)*0.5-0.25,-0.3,0))
            front =[*l1.get_points()]

        line2 = Line((np.sin(PI/3)*0.5-0.25,0,0),(0.5,0,0))
        self.append_points(line1.get_points())
        self.append_points(tri.get_points())
        self.append_points(front)
        self.append_points(line2.get_points())


class VoltageSource(CircuitComponent):
    CONFIG = {
    "is_dependent" : False
    }

    def __init__(self,**kwargs):
        VMobject.__init__(self,**kwargs)

    def generate_points(self):

        if(self.is_dependent):
            neg = Line((-0.5,0,0),(-0.35,0,0))
            neg_label = Line((-0.15,0.05,0),(-0.15,-0.05,0))
            pos_label1 = Line((0.15,0.05,0),(0.15,-0.05,0))
            pos_label2 = Line((0.1,0,0),(0.2,0,0))
            mid = Square(side_length=0.5)
            mid.rotate(PI/4)
            pos = Line((0.35,0,0),(0.5,0,0))        
        else:
            neg = Line((-0.5,0,0),(-0.25,0,0))
            neg_label = Line((-0.15,0.05,0),(-0.15,-0.05,0))
            pos_label1 = Line((0.15,0.05,0),(0.15,-0.05,0))
            pos_label2 = Line((0.1,0,0),(0.2,0,0))
            mid = Circle(radius=0.25)
            pos = Line((0.25,0,0),(0.5,0,0))

        self.append_points(neg.get_points())
        self.append_points(mid.get_points())
        self.append_points(neg_label.get_points())
        self.append_points(pos_label1.get_points())
        self.append_points(pos_label2.get_points())
        self.append_points(pos.get_points())



class CurrentSource(CircuitComponent):
    CONFIG = {
    "is_dependent" : False
    }

    def __init__(self,**kwargs):
        VMobject.__init__(self,**kwargs)


    def generate_points(self):

        if(self.is_dependent):
            neg = Line((-0.5,0,0),(-0.35,0,0))
            arr = Vector((0.3,0,0))
            arr.shift(LEFT*0.15)
            mid = Square(side_length=0.5)
            mid.rotate(PI/4)
            pos = Line((0.35,0,0),(0.5,0,0))           
        else:
            neg = Line((-0.5,0,0),(-0.25,0,0))
            arr = Vector((0.3,0,0))
            arr.shift(LEFT*0.15)
            mid = Circle(radius=0.25)
            pos = Line((0.25,0,0),(0.5,0,0))

        self.append_points(neg.get_points())
        self.append_points(mid.get_points())
        self.append_points(arr.get_points())
        self.append_points(arr.get_tip().get_points())
        self.append_points(pos.get_points())

class ACSource(CircuitComponent):

    def generate_points(self):

        verts = [(-1.5,-0.5,0),(0.3642-1.5,-0.5,0),(0.6358-1.5,0.5,0),(1-1.5,0.5,0),(1-1.5,0.5,0),(1.3634-1.5,0.5,0),(1.6358-1.5,-0.5,0),(2-1.5,-0.5,0),(2-1.5,-0.5,0),(2.3642-1.5,-0.5,0),(2.6358-1.5,0.5,0),(1.5,0.5,0)]
        sinewave = CubicBezier(verts)
        sinewave.rotate(-PI/2)
        sinewave.scale(0.1)
        mid = Circle(radius=0.25)
        neg = Line((-0.5,0,0),(-0.25,0,0))
        pos = Line((0.25,0,0),(0.5,0,0))

        self.append_points(neg.get_points())
        self.append_points(mid.get_points())
        self.append_points(sinewave.get_points())
        self.append_points(pos.get_points())


#Transistors
class Bjt(CircuitComponent):
    CONFIG = {
    "is_npn" : True,
    "collector_loc" : 0,
    "emitter_loc" : 0
    }

    

    def __init__(self,**kwargs):
        VMobject.__init__(self, **kwargs)
        

    def generate_points(self):

        
        base = Line((-0.5,0,0),(0,0,0))
        wall = Line((0,0.5,0),(0,-0.5,0))

        e_coord = (0,0,0)

        if(self.is_npn):
            emitter = Vector(direction = (0.5,-0.375,0))
            collector = Line((0,0.125,0),(0.5,0.5,0))
            emitter.shift(0.125*DOWN)

            e_coord = emitter.get_tip().get_points()[len(emitter.get_tip().get_points()) - 1]
        else:
            emitter = Vector(direction = (-0.5,-0.375,0))
            collector = Line((0,-0.125,0),(0.5,-0.5,0))
            emitter.shift(0.5*RIGHT + UP*0.5)

            e_coord = emitter.get_points()[0]

        
        c_coord = collector.get_points()[len(collector.get_points()) - 1]


        self.append_points(base.get_points())
        self.append_points(wall.get_points())
        self.append_points(collector.get_points())
        self.append_points(emitter.get_points())
        self.append_points(emitter.get_tip().get_points())

        #Getting point locations
        idx = 0
        for coord in self.get_points():
            if coord[0] == c_coord[0] and coord[1] == c_coord[1] and coord[2] == c_coord[2]:
                self.collector_loc = idx

            if coord[0] == e_coord[0] and coord[1] == e_coord[1] and coord[2] == e_coord[2]:
                self.emitter_loc = idx

            idx += 1




    def get_base(self):
        return self.get_points()[0]

    def get_emitter(self):
        return self.get_points()[self.emitter_loc]

    def get_collector(self):
        return self.get_points()[self.collector_loc]







class Mosfet(CircuitComponent):
    CONFIG = {
    "is_nmos" : True,
    "drain_loc" : 0,
    "source_loc" : 0
    }

    def __init__(self,**kwargs):
        VMobject.__init__(self,**kwargs)


    def generate_points(self):


        gate = Line((-0.5,0,0),(-0.125,0,0))
        wall1 = Line((-0.125,0.4,0),(-0.125,-0.4,0))
        wall2 = Line((0.125,0.5,0),(0.125,-0.5,0))
        top_line = Line((0.125,0.4,0),(0.5,0.4,0))
        bot_line = Line((0.125,-0.4,0),(0.5,-0.4,0))


        verts = []

        if(self.is_nmos):
            body = Vector(direction=(-0.375,0,0))
            body.shift(RIGHT*0.5)
            drain = Line((0.5,0.4,0),(0.5,0.75,0))
            source = Line((0.5,0,0),(0.5,-0.75,0))

            verts = [*bot_line.get_points(),*top_line.get_points(),*drain.get_points(),*body.get_tip().get_points(),*body.get_points()[::-1],*source.get_points()]
        else:
            body = Vector(direction=(0.375,0,0))
            body.shift(RIGHT*0.125)
            drain = Line((0.5,-0.4,0),(0.5,-0.75,0))
            source = Line((0.5,0,0),(0.5,0.75,0))

            verts = [*top_line.get_points(),*bot_line.get_points(),*drain.get_points(),*body.get_points(),*body.get_tip().get_points(),*source.get_points()]


        s_coord = source.get_points()[len(source.get_points()) - 1]
        d_coord = drain.get_points()[len(drain.get_points()) - 1]



        self.append_points(gate.get_points())
        self.append_points(wall1.get_points())
        self.append_points(wall2.get_points())

        self.append_points(verts)
        
        


        idx = 0
        for coord in self.get_points():

            if coord[0] == s_coord[0] and coord[1] == s_coord[1] and coord[2] == s_coord[2]:
                self.source_loc = idx

            if coord[0] == d_coord[0] and coord[1] == d_coord[1] and coord[2] == d_coord[2]:
                self.drain_loc = idx

            idx += 1


    def get_gate(self):
        return self.get_points()[0]

    def get_source(self):
        return self.get_points()[self.source_loc]

    def get_drain(self):
        return self.get_points()[self.drain_loc]





import inspect
import itertools as it
import os
import platform
import subprocess as sp
import sys
import traceback

from manimlib.scene.scene import Scene
from manimlib.utils.sounds import play_error_sound
from manimlib.utils.sounds import play_finish_sound
import manimlib.constants


def open_file_if_needed(file_writer, **config):
    if config["quiet"]:
        curr_stdout = sys.stdout
        sys.stdout = open(os.devnull, "w")

    open_file = any([
        config["open_video_upon_completion"],
        config["show_file_in_finder"]
    ])
    if open_file:
        current_os = platform.system()
        file_paths = []

        if config["file_writer_config"]["save_last_frame"]:
            file_paths.append(file_writer.get_image_file_path())
        if config["file_writer_config"]["write_to_movie"]:
            file_paths.append(file_writer.get_movie_file_path())

        for file_path in file_paths:
            if current_os == "Windows":
                os.startfile(file_path)
            else:
                commands = []
                if current_os == "Linux":
                    commands.append("xdg-open")
                elif current_os.startswith("CYGWIN"):
                    commands.append("cygstart")
                else:  # Assume macOS
                    commands.append("open")

                if config["show_file_in_finder"]:
                    commands.append("-R")

                commands.append(file_path)

                # commands.append("-g")
                FNULL = open(os.devnull, 'w')
                sp.call(commands, stdout=FNULL, stderr=sp.STDOUT)
                FNULL.close()

    if config["quiet"]:
        sys.stdout.close()
        sys.stdout = curr_stdout


def is_child_scene(obj, module):
    if not inspect.isclass(obj):
        return False
    if not issubclass(obj, Scene):
        return False
    if obj == Scene:
        return False
    if not obj.__module__.startswith(module.__name__):
        return False
    return True


def prompt_user_for_choice(scene_classes):
    num_to_class = {}
    for count, scene_class in zip(it.count(1), scene_classes):
        name = scene_class.__name__
        print("%d: %s" % (count, name))
        num_to_class[count] = scene_class
    try:
        user_input = input(manimlib.constants.CHOOSE_NUMBER_MESSAGE)
        return [
            num_to_class[int(num_str)]
            for num_str in user_input.split(",")
        ]
    except KeyError:
        print(manimlib.constants.INVALID_NUMBER_MESSAGE)
        sys.exit(2)
        user_input = input(manimlib.constants.CHOOSE_NUMBER_MESSAGE)
        return [
            num_to_class[int(num_str)]
            for num_str in user_input.split(",")
        ]
    except EOFError:
        sys.exit(1)


def get_scenes_to_render(scene_classes, config):
    if len(scene_classes) == 0:
        print(manimlib.constants.NO_SCENE_MESSAGE)
        return []
    if config["write_all"]:
        return scene_classes
    result = []
    for scene_name in config["scene_names"]:
        found = False
        for scene_class in scene_classes:
            if scene_class.__name__ == scene_name:
                result.append(scene_class)
                found = True
                break
        if not found and (scene_name != ""):
            print(
                manimlib.constants.SCENE_NOT_FOUND_MESSAGE.format(
                    scene_name
                ),
                file=sys.stderr
            )
    if result:
        return result
    return [scene_classes[0]] if len(scene_classes) == 1 else prompt_user_for_choice(scene_classes)


def get_scene_classes_from_module(module):
    if hasattr(module, "SCENES_IN_ORDER"):
        return module.SCENES_IN_ORDER
    else:
        return [
            member[1]
            for member in inspect.getmembers(
                module,
                lambda x: is_child_scene(x, module)
            )
        ]


def main(config):
    module = config["module"]
    all_scene_classes = get_scene_classes_from_module(module)
    scene_classes_to_render = get_scenes_to_render(all_scene_classes, config)

    scene_kwargs = dict([
        (key, config[key])
        for key in [
            "camera_config",
            "file_writer_config",
            "skip_animations",
            "start_at_animation_number",
            "end_at_animation_number",
            "leave_progress_bars",
        ]
    ])

    for SceneClass in scene_classes_to_render:
        try:
            # By invoking, this renders the full scene
            scene = SceneClass(**scene_kwargs)
            open_file_if_needed(scene.file_writer, **config)
            if config["sound"]:
                play_finish_sound()
        except Exception:
            print("\n\n")
            traceback.print_exc()
            print("\n\n")
            if config["sound"]:
                play_error_sound()


if __name__ == "__main__":
    main()
